@@ -96,6 +96,7 @@ debug_calls uses dynamic =..  .
 @version 1.4 2020/9/18
 @version 1.5 2022/12/29
 @version 2.0 2025/10/7
+@version 2.1 2025/10/27
 @see debug_call/4 for version information
 
 */
@@ -106,11 +107,11 @@ Current version and release date for the library.
 
 ==
 ?- debug_call_version( -V, -D ).
-V = 1:5:0,
-D = date(2022,12,29).
+V = 2:1:0,
+D = date(2025,10,27).
 ==
 */
-debug_call_version( 1:5:0, date(2022,12,29) ).
+debug_call_version( 2:1:0, date(2025,10,27) ).
 
 :- use_module(library(apply)).   % maplist/4,...
 :- use_module(library(lists)).   % member/4,...
@@ -376,9 +377,9 @@ debug_portray( _Topic, _Term ).
 %% debug_call( +Topic, +Goal, +Arg ).
 %% debug_call( +Topic, +Goal, +Arg, +Opts ).
 %
-% Automates often used debug calls with emphasis on avoiding calling things that will not be reported and tailoring the messages.
+% Automates often used debug calls with emphasis on: (a) avoiding calling things that will not be reported and (b) easy tailoring of the messages.
 % 
-% The main novelty is the introduction of abbreviated Goals, that print bespoke message for often used debugging information. 
+% The main novelty is the introduction of abbreviated Goals, that print bespoke messages for often used debugging information. 
 % For example the following code ejects info on the legth of the list. Not only the code for calculating the length
 % only happens if debugging for the topic ex, is on, but the message is also tailored to reporting lengths of lists.
 %==
@@ -386,28 +387,54 @@ debug_portray( _Topic, _Term ).
 % ?- debug_call(ex, length, math_vars/[x,y,z]).
 % % Length for list, math_vars: 3
 %==
+% With v1.3 the debuc/n shorthand was introduced. So debuc/1,2,3,4 are shorthands for debug_call/1,2,3,4.
+% 
+% ==
+% ?- Mtx = [h(a,b,c),r(1,2,3),r(4,5,6),r(7,8,9)],
+%    debuc(ex, dims, mtx/Mtx).
+% 
+% Dimensions for matrix, mtx: nR: 4, nC: 3.
+% ==
+% 
+% The predicate can work as a replacement to debug/3. That is, if Goal does not match any of the forms below, it will be interpreted as a message.
+% ==
+% ?- debuc(ex, 'A simple message in a ~a.', [bottle] ).
+% A simple message in a bottle.
+% ==
+% 
+% The predicate can be used to call arbitrary Goal and then print a message after it has successfull completed (see below).<br>
+% When Goal is a known abbreviation from those shown below, the Arg usually qualifies the output generated.
 %
-% Predicate can be used to call arbitrary Goal and then print a message after it has successfull completed. <br>
-%  When Goal is a known abbreviation from those shown below, the Arg usually qualifies the output generated.
-%  When Goal is of the form call(Goal), Arg will be passed to debug(Topic,Mess,Arg). 
-%
-%  As of v2 the last two arguments of the /4 version of the predicate where switched from _Pfx_ and Arg
+%  As of v2 the last two arguments of the /4 version of the predicate were switched from _Pfx_ and Arg
 %  to Arg and Opts. Opts pass arbitary things to Goal, each abbreviation Goal can demand different options. 
-%  All abbreviation Goal them can take =prefix(Pfx)= which corresponds to Pfx in the old /4 verison. 
-%  Opts will be forced to be a list via en_list/2, Goal will know what todo with it.
-% 
-% As of v1.2 it can work as a replacement to debug/3. <br>
-% With v1.3 the debuc/3 shorthand was introduced.
-% 
+%  All debuc Goals can take =|prefix(Pfx)|= which corresponds to Pfx in the old /4 version, and =|pred(Fnc,Ar)|= or =|pred(Pid)|=.
+%==
+% ?- debuc(ex, enum, list_x/[x1,x1,x3], [pred(integral,2),prefix('At')] ).
+% % At predicate: integral/2 starting enumeration of list: list_x
+% % 1.x1
+% % 2.x1
+% % 3.x3
+% % At predicate: integral/2 ended enumeration of list: list_x
+%==
+% The predicate is relaxed about Opts. It can be a single term, which will be cast into a list.
+%==
+% ?- debuc(ex, pwd, my_run, pred(bio_db,3) ).
+%
+% Predicate: bio_db/3 pwd at, my_run, is: '/home/nicos/pl/packs/private/debug_call/'
+%==
+%
 % Goal in:
 %  * call(Goal)
-%    call Goal before printing debugging message debug(Topic, Mess, Arg).  (Goal is called in non-deterministic context.)
+%    call Goal before printing debugging message debug(Topic, Mess, MArgS).  Goal is called in deterministic context.
+%    Goal is called with extra arguments +Arg, -Mess and -MArgS.
+%  * call(Goal,Opts) 
+%    as above, but Opts are passed as an extra, last argument in the call
 %  * dims
 %    prints the dimensions of matrix, see mtx_dims/3
 %  * end
 %    translates to finishing ~Arg or finishing ~Topic if Arg == true
 %  * enum
-%    print lists and deconstructed terms, where each item is prefixed with an index number
+%    print members of lists and arguments of terms, where each item is printed on single line and prefixed by an index number
 %  * goal
 %    anything that does n't match any of the above is retrived as call(Goal)
 %  * info
@@ -426,18 +453,15 @@ debug_portray( _Topic, _Term ).
 %    else it is translated as Hdr/''/List or ''/''/List. 
 %    If Hdr or Ftr are '' then that part of the message is skipped
 %  * ns_sel
-%    first argument is the item selected from second arg list (only reported if 2nd arg is not a singleton (ns))
-%    accepts 2 optional args, 3rd is the token of what is selected (false for printing nothing on the subject, default)
-%    and 4th is whether to report if the 2nd argument is indeed a singleton (default: false)
-%  * ns_sel(true)
-%    first argument is the item selected from second arg list. reports differently if 2nd arg is a singleton, but always does report
+%    first argument of Arg is the item selected from second arg which is expected to be a list.
+%    The selected argument can be named on the massage via sel_name(Lnm) in Opts.
 %  * odir
 %    output directory (Arg should exist and be a directory)
 %  * option
-%    option selected from options for predicate. Possible options: pred(Pid), the caller predicate, all(OrigOpts), shows all options, 
+%    option selected from options for predicate. Possible options: pred(Fnc,Ar) or pred(Pid), the caller predicate, all(OrigOpts), shows all options, 
 %    internal(true), shows also '$' starting options.
 %  * options
-%    options used on call to a predicate. Possible options: pred(Pid), the caller predicate, internal(true), shows also '$' starting options.
+%    options used on call to a predicate. Possible options: pred(Func,Ar), pred(Pid), the caller predicate, internal(true), shows also '$' starting options.
 %  * pwd 
 %    message about current dir Location (=Arg), (if Arg == false, location is not shown)- see examples
 %  * read
@@ -447,7 +471,7 @@ debug_portray( _Topic, _Term ).
 %  * task(Wch)  
 %    time of start/stop (Wch) of a task. Other values for Wch are allowed but printed as they come. Arg can be a term (as of Version 1.5).
 %  * term
-%    simply spew the input term
+%    Report the input term. The term can be named via option term_name(Tnm).
 %  * var
 %    reports variable name (arg(1)) and its current instantiation (arg(2))
 %  * wrote 
@@ -455,7 +479,9 @@ debug_portray( _Topic, _Term ).
 %    Either loc(File,Exts) or simply File in which case Exts = ''.
 %    As of v2.0 the default is to print the basename, use path(abs) in Opts.
 %
-% See file examples/exo.pl for a test suit including at least one example from each shorthand call.
+% As of v2.1 all debuc Goals work with options prefix(Pfx) and pred(Ar,Fn) (also synonymed to pred(Pid)).
+% 
+% See file examples/exo.pl for a test suit including at least one example from each debuc Goal.
 %
 %==
 % ?- debug(ex).
@@ -472,8 +498,8 @@ debug_portray( _Topic, _Term ).
 % 
 % ?- debuc(ex, wrote, loc(file,csv)).
 % % Could not locate wrote on file specified by: file, and extensions: csv
-% ?- csv_write_file( 'file.csv', []).
 %
+% ?- csv_write_file( 'file.csv', []).
 % ?- debuc(ex, wrote, loc(file,csv)).
 % % Wrote on file: 'file.csv'
 %
@@ -492,26 +518,35 @@ debug_portray( _Topic, _Term ).
 % true.
 % 
 % ?-  Etcs = [suv-17.09.26.txg,suv-17.09.21.txg], Etc = suv-17.09.26.txg,
-%     debuc(suv, ns_sel, c(Etc,Etcs,'suv file',true)).
-% Continuing with: suv file, as: suv-17.09.26.txg, from non singleton list: [suv-17.09.26.txg,suv-17.09.21.txg]
+%     debuc(suv, ns_sel, c(Etc,Etcs, sel_name('suv file') ).
+% Continuing with: suv-17.09.26.txg as the: suv file. From list: [suv-17.09.26.txg,suv-17.09.21.txg]
 %
+% ?- assert( (list_avg_mess(List,Mess,Args) :- length(List,Len), sum_list(List,Sum), Avg is Sum / Len, Mess = 'Avg: ~w', Args = Avg) ).
+% ?- debuc( ex, call(list_avg_mess), [1,2,3] ).
+% Avg: 2
+%
+% ?- debuc( ex, call(list_avg_mess), [1,2,3], prefix('By call') ).
+% By call avg: 2
+%
+% ?- debuc( ex, call(list_avg_mess), [1,2,3], [pred(p1,2),prefix('By call')] ).
+% By call predicate: p1/2 avg: 2
 %==
 % 
 % At some point around SWI-Prolog 8, behaviour of debug/3 changed in being more strict about messages with no arguments.
 % As of version 1.2 debug_call/3 can act as a replacement of debug/3 but with the old behaviour.
 %
 %==
-% ?- debug( ex, 'Messagging...', true ).
-% Messagging...
-% [[ EXCEPTION while printing message 'Messagging...'
+% ?- debug( ex, 'Messaging...', true ).
+% Messaging...
+% [[ EXCEPTION while printing message 'Messaging...'
 %       with arguments user:true:
 %       raised: format('too many arguments')
 %    ]]
 % 
 % true.
 % 
-% ?- debuc( ex, 'Messagging...', true ).
-% % Messagging...
+% ?- debuc( ex, 'Messaging...', true ).
+% % Messaging...
 % true.
 %==
 %
@@ -524,7 +559,8 @@ debug_portray( _Topic, _Term ).
 % @version  1.1 2018/03/20  prefer +2 arity in debug_call/2
 % @version  1.2 2020/03/07  now can be used as a replacement for debug/3 (but with old 3rd arg behaviour, allowing eg 'true').
 % @version  1.3 2020/09/14  added canned calls info and enum, debuc/2,3,4
-% @version  2.0 2025/10/07  changed last two arguments, new option goal recogniser, pred/1, internal/1 & all/1 [not published yet]
+% @version  2.0 2025/10/07  changed last two arguments, new option goal recogniser, pred/1, internal/1 & all/1
+% @version  2.1 2025/10/27  pid(F,A) & prefix() universal; call() fixed; doc; enum terms fix; ns_sel simplify
 % @see file examples/exo.pl
 % @see debuc/3 shorthand for debug_call/3
 %
@@ -538,13 +574,20 @@ debug_call( Topic, Goal, Arg, OptsPrv ) :-
     debugging_call( Topic, Goal, Arg, Opts ).
 debug_call( _Topic, _Goal, _Arg, _Opts ).
 
+debugging_call( Topic, call(Goal), Arg, Opts) :-
+    !,
+    call( Goal, Arg, Gess, Grgs ),
+    !,
+    debug_call_message_opts( Gess, Grgs, Mess, Args, Opts ),
+    debug_message( Topic, Mess, Args ).
+debugging_call( Topic, call_opts(Goal), Arg, Opts ) :-
+    !,
+    call( Goal, Arg, Gess, Grgs, Opts ),
+    debug_call_message_opts( Gess, Grgs, Mess, Args, Opts ),
+    debug_message( Topic, Mess, Args ).
 debugging_call( Topic, Goal, Arg, Opts ) :- 
     debug_call_topic( Goal, Arg, Opts, Topic ),
     !.
-debugging_call( Topic, call(Goal), Mess, Args ) :-
-    !,
-    call( Goal ),
-    debug_message( Topic, Mess, Args ).
 debugging_call( Topic, Goal, Mess, Args ) :-
     compound( Goal ),
     call( Goal ),
@@ -627,64 +670,64 @@ debug_consec_color( Topic, Clr, Mess, Args ) :-
     retractall( debug_call_message_property(debug(_),color(_)) ).
 
 debug_call_topic( info, Arg, Bogs, _Topic ) :-
-    ( (\+ var(Arg),Arg = Mess/Args) ->
-        true
-        ;
-        % fixme: not sure what to do here ?
-        Mess = Arg,
-        Args = []
-    ),
-    % lib_message_report( Format, Args, Kind ) :-
-     debug_message_prefixed( Bogs, Mess, Prefixed ),
-	phrase('$messages':translate_message(debug(Prefixed,Args)), Lines),
+     ( (\+ var(Arg),Arg = Mess/Args) ->
+          true
+          ;
+          % fixme: not sure what to do here ?
+          Mess = Arg,
+          Args = []
+     ),
+     debug_call_message_opts( Mess, Args, Prefixed, Prgs, Bogs ),
+	phrase('$messages':translate_message(debug(Prefixed,Prgs)), Lines),
 	print_message_lines(current_output, kind(informational), Lines).
 debug_call_topic( dims, NamesPrv/MtxsPrv, Bogs, Topic ) :-
     ( is_list(NamesPrv) -> Names=NamesPrv, MtxsPrv=Mtxs, With = 'Dimensions for matrices, '
                            ; [NamesPrv] = Names, [MtxsPrv]=Mtxs, With = 'Dimensions for matrix, ' 
     ),
-    debug_message_prefixed( Bogs, With, Prefixed ),
     maplist( debug_mtx_dims, Mtxs, NRows, NCols ),
-    findall( PartM, (member(_,Names),PartM=' (~w) nR: ~d, nC: ~d.'), MParts ),
+    findall( PartM, (member(_,Names),PartM='~w: nR: ~d, nC: ~d.'), MParts ),
     atomic_list_concat( MParts, '', Right ),
     findall( [Name,NRow,NCol], (nth1(N,Names,Name),nth1(N,NRows,NRow),nth1(N,NCols,NCol)), NNest ),
-    flatten( NNest, Vars ),
-    atom_concat( Prefixed, Right, Message ),
-    debug_message( Topic, Message, Vars ). % do the messaging !
+    flatten( NNest, Vargs ),
+    atom_concat( With, Right, Vess ),
+    debug_call_message_opts( Vess, Vargs, Message, Args, Bogs ),
+    debug_message( Topic, Message, Args ).
 debug_call_topic( enum, InArg, Bogs, Topic ) :-
-    ( memberchk(prefix(Pfx),Bogs) -> true; Pfx = '' ),
     ground( InArg ),
     ( InArg = Left/Term -> true; Left = unnamed, Term = InArg ),
     ( is_list(Term) ->
         length( Term, Len ),
         number_codes( Len, LenCs ),
         length( LenCs, SpcLen ),
-        debug_call_topic_list_delim( Left, Topic, prefix(Pfx), 'Starting enumeration of list: ~w' ),
+        debug_call_topic_list_delim( Left, Topic, 'Starting enumeration of list: ~w', Bogs ),
         debug_call_topic_enum( Term, 1, SpcLen, Topic ),
-        debug_call_topic_list_delim( Left, Topic, prefix(Pfx), 'Ended enumeration of list: ~w' )
+        debug_call_topic_list_delim( Left, Topic, 'Ended enumeration of list: ~w', Bogs )
         ;
-        Term =.. Args,
+        Term =.. [Func|Args],
         length( Args, Len ),
         number_codes( Len, LenCs ),
         length( LenCs, SpcLen ),
-        debug_call_topic_list_delim( Left, Topic, prefix(Pfx), 'Starting enumeration of list: ~w' ),
+        atomic_list_concat( ['Starting enumeration of term: ~w (func: ',Func,')'], StrMess ),
+        debug_call_topic_list_delim( Left, Topic, StrMess, Bogs ),
         debug_call_topic_enum( Args, 1, SpcLen, Topic ),
-        debug_call_topic_list_delim( Left, Topic, prefix(Pfx), 'Ended enumeration of list: ~w' )
+        atomic_list_concat( ['Ended enumeration of term: ~w (func: ',Func,')'], EndMess ),
+        debug_call_topic_list_delim( Left, Topic, EndMess, Bogs )
     ).
 debug_call_topic( length, NamesPrv/ListsPrv, Bogs, Topic ) :-
     ( is_list(NamesPrv) -> Names=NamesPrv, ListsPrv=Lists, With = 'Lengths for lists, '
                            ; [NamesPrv] = Names, [ListsPrv]=Lists, With = 'Length for list, ' 
     ),
-    debug_message_prefixed( Bogs, With, Prefixed ),
     maplist( length, Lists, Lengths ),
     findall( ['~w: ~w',', '], member(_,Lengths), WsNest ),
     flatten( WsNest, WsL ),
     once( append(WsLComma,[_],WsL) ),
     append( WsLComma, ['.'], WsLDot ),
     atomic_list_concat( WsLDot, '', Right ),
-    atom_concat( Prefixed, Right, Message ),
     findall( [Name,Length], (nth1(N,Names,Name),nth1(N,Lengths,Length)), NLNest ),
     flatten( NLNest, NLs ),
-    debug_message( Topic, Message, NLs ). % do the messaging
+    atom_concat( With, Right, Vess ),
+    debug_call_message_opts( Vess, NLs, Message, Args, Bogs ),
+    debug_message( Topic, Message, Args ). % do the messaging
 debug_call_topic( list, InArg, Bogs, Topic ) :-
     ground( InArg ),
     ( InArg = Left/List -> 
@@ -692,137 +735,111 @@ debug_call_topic( list, InArg, Bogs, Topic ) :-
         ;
         List = InArg, Hdr = '', Ftr = ''
     ),
-    ( memberchk(prefix(Pfx),Bogs) -> true; Pfx = '' ),
-    debug_call_topic_list_delim( Hdr, Topic, Pfx, 'Starting listing of list: ~w' ),
+    debug_call_topic_list_delim( Hdr, Topic, 'Starting listing of list: ~w', Bogs),
     maplist( debug_message(Topic,'~w'), List ),
-    debug_call_topic_list_delim( Ftr, Topic, Pfx, 'Ended listing of list: ~w' ).
+    debug_call_topic_list_delim( Ftr, Topic, 'Ended listing of list: ~w', Bogs ).
 debug_call_topic( odir, Odir, Bogs, Topic ) :-
     ( exists_directory(Odir) ->
         Mess = 'Output in directory: ~w'
         ;
         Mess = 'Output (claimed) in (non-existing) directory: ~w'
     ),
-    debug_message_prefixed( Bogs, Mess, Prefixed ),
-    debug_message( Topic, Prefixed, [Odir] ).
+    debug_call_message_opts( Mess, [Odir], Message, Args, Bogs ),
+    debug_message( Topic, Message, Args ).
 debug_call_topic( option, Opt, Bogs, Topic ) :-
-    ( memberchk(pred(Pid),Bogs) ->
-          Pess = 'Predicate: ~w, option selected: ~w',
-          Prgs = [Pid,Opt]
-          ;
-          Pess = 'Option selected: ~w',
-          Prgs = [Opt]
-    ),
+    Ness = 'Option selected: ~w',
     ( (memberchk(all(OrgOpts),Bogs),is_list(OrgOpts)) ->
                ( memberchk(internal(true),Bogs) ->
                     RdcOpts = OrgOpts
                     ;
                     findall( R, (member(R,OrgOpts),functor(R,F,_),\+(atom_concat('$',_,F))), RdcOpts )
                ),
-               atom_concat( Pess, ' from options: ~w', Mess ),
-               append( Prgs, [RdcOpts], Mrgs )
+               atom_concat( Ness, ' from options: ~w', Mess ),
+               Mrgs = [Opt,RdcOpts]
                ;
-               atom_concat( Pess, '.', Mess ),
-               Prgs = Mrgs
+               atom_concat( Ness, '.', Mess ),
+               [Opt] = Mrgs
     ),
-    debug_message_prefixed( Bogs, Mess, Prefixed ),
-    debug_message( Topic, Prefixed, Mrgs ).
+    debug_call_message_opts( Mess, Mrgs, Message, Args, Bogs ),
+    debug_message( Topic, Message, Args ).
 debug_call_topic( options, RepOpts, Bogs, Topic ) :-
-    ( memberchk(pred(Pid),Bogs) ->
-          Mess = 'Predicate: ~w, with options: ~w.',
-          Mrgs = [Pid,RdcOpts]
-          ;
-          Mess = 'Options: ~w.',
-          Mrgs = [RdcOpts]
-    ),
-    debug_message_prefixed( Bogs, Mess, Prefixed ),
+    Ness = 'Options: ~w',
     ( memberchk(internal(true),Bogs) -> 
                RepOpts = RdcOpts
                ;
                findall( R, (member(R,RepOpts),functor(R,F,_),\+(atom_concat('$',_,F))), RdcOpts )
     ),
-    debug( Topic,  Prefixed, Mrgs ).
-debug_call_topic( term, DbgTerm, Bogs, Topic ) :-
-    Mess = '~w',
-    debug_message_prefixed( Bogs, Mess, Prefixed ),
-    debug_message( Topic, Prefixed, [DbgTerm] ).
+    debug_call_message_opts( Ness, [RdcOpts], Message, Args, Bogs ),
+    debug( Topic,  Message, Args ).
+debug_call_topic( term, Derm, Bogs, Topic ) :-
+    ( memberchk(term_name(Tnm),Bogs) -> 
+          Mess = 'Reporting term (~w): ~w',
+          Mrgs = [Tnm,Derm]
+          ; 
+          Mess = 'Reporting term: ~w',
+          Mrgs = [Derm]
+    ),
+    debug_call_message_opts( Mess, Mrgs, Message, Args, Bogs ),
+    debug_message( Topic, Message, Args ).
 debug_call_topic( var, DbgTerm, Bogs, Topic ) :-
     arg( 1, DbgTerm, Var ),
     arg( 2, DbgTerm, Val ),
     Mess = 'Variable: ~a, value: ~w',
-    debug_message_prefixed( Bogs, Mess, Prefixed ),
-    debug_message( Topic, Prefixed, [Var,Val] ).
+    debug_call_message_opts( Mess, [Var,Val], Message, Args, Bogs ),
+    debug_message( Topic, Message, Args ).
 debug_call_topic( wrote, ForLoc, Bogs, Topic ) :-
     ( ForLoc = loc(Spec,Ext) -> true; Spec=ForLoc, Ext = '' ),
     catch( locate(Spec,Ext,Loc), Excp, true ),
     MessW = 'Wrote on file: ~p',
-    debug_call_location_exception_message( Excp, write, Loc, MessW, Mess, Bogs, Args ),
-    debug_message_prefixed( Bogs, Mess, Prefixed ),
-    debug_message( Topic, Prefixed, Args ).
+    debug_call_location_exception_message( Excp, write, Loc, MessW, Mess, Bogs, Mrgs ),
+    debug_call_message_opts( Mess, Mrgs, Message, Args, Bogs ),
+    debug_message( Topic, Message, Args ).
 debug_call_topic( read, ForLoc, Bogs, Topic ) :-
      debug_call_topic( input, ForLoc, Bogs, Topic ).
 debug_call_topic( input, ForLoc, Bogs, Topic ) :-
     ( ForLoc = loc(Spec,Ext) -> true; Spec=ForLoc, Ext = '' ),
     catch( locate(Spec,Ext,Loc), Excp, true ),
     MessW = 'Input from file: ~p',
-    debug_call_location_exception_message( Excp, input, Loc, MessW, Mess, Bogs, Args ),
-    debug_message_prefixed( Bogs, Mess, Prefixed ),
-    debug_message( Topic, Prefixed, Args ).
+    debug_call_location_exception_message( Excp, input, Loc, MessW, Mess, Bogs, Mrgs ),
+    debug_call_message_opts( Mess, Mrgs, Message, Args, Bogs ),
+    debug_message( Topic, Message, Args ).
 debug_call_topic( task(Whc), Task, Bogs, Topic ) :-
     datime_readable( Readable ),
     debug_call_topic_time_which_readable( Whc, Whcable ),
     atomic_list_concat( [Readable,' ',Whcable,' task: ~w.'], Mess ),
-    debug_message_prefixed( Bogs, Mess, Prefixed ),
-    debug_message( Topic, Prefixed, [Task] ).
+    debug_call_message_opts( Mess, [Task], Message, Args, Bogs ),
+    debug_message( Topic, Message, Args ).
 debug_call_topic( start, Arg, Bogs, Topic ) :-
     Mess = 'Starting: ~w',
-    debug_message_prefixed( Bogs, Mess, Prefixed ),
     ( Arg == true -> Rep = Topic; Rep = Arg ),
-    debug_message( Topic, Prefixed, [Rep] ).
-% 25.10.07, use this as an example of how to have back compatibility to the new args order.
+    debug_call_message_opts( Mess, [Rep], Message, Args, Bogs ),
+    debug_message( Topic, Message, Args ).
 debug_call_topic( end, Arg, Bogs, Topic ) :-
     Mess = 'Finished: ~w',
-    debug_message_prefixed( Bogs, Mess, Prefixed ),
     ( Arg == true -> Rep = Topic; Rep = Arg ),
-    debug_message( Topic, Prefixed, [Rep] ).
+    debug_call_message_opts( Mess, [Rep], Message, Args, Bogs ),
+    debug_message( Topic, Message, Args ).
 debug_call_topic( pwd, Stage, Bogs, Topic ) :-
     working_directory( Pwd, Pwd ),
     ( Stage == false -> 
-        Mess = 'Pwd: ~p', Args = [Pwd]
+        Mess = 'Pwd: ~p', Mrgs = [Pwd]
         ;
-        Mess = 'Pwd at, ~w, is: ~p', Args = [Stage,Pwd]
+        Mess = 'Pwd at, ~w, is: ~p', Mrgs = [Stage,Pwd]
     ),
-    debug_message_prefixed( Bogs, Mess, Prefixed ),
-    debug_message( Topic, Prefixed, Args ).
+    debug_call_message_opts( Mess, Mrgs, Message, Args, Bogs ),
+    debug_message( Topic, Message, Args ).
 debug_call_topic( ns_sel, Term, Bogs, Topic ) :-
-    % ( Term = [Fst,Sec] -> true; arg(1,Term,Fst),arg(2,Term,Sec) ),
-    arg( 1, Term, Fst ), 
+    arg( 1, Term, Fst ),
     arg( 2, Term, Sec ),
-    functor( Term, _Tname, Arity ),
-    ( Sec == [] -> 
-        true % fixme: it will make more sense to throw an error if Sec = []
-        ;
-        ( Sec = [_Single] ->
-            ( (Arity>3,arg(4,Term,true)) ->
-                ( (Arity>2,\+ arg(3,Term,false)) ->
-                    arg(3,Term,Trd),
-                    Mess= 'Continuing with: ~w as: ~w, (only match).', MArgs = [Trd,Fst]
-                    ;
-                    Mess= 'Continuing with only match: ~w.', MArgs = [Fst,Sec]
-                )
-                ;
-                Mess = 'Continuing: ~w, from non singleton list: ~w', MArgs = [Fst,Sec]
-            )
-            ;
-            ( (Arity>2,\+ arg(3,Term,false)) ->
-                arg(3,Term,Trd),
-                Mess = 'Continuing with: ~w, as: ~w, from non singleton list: ~w', MArgs = [Trd,Fst,Sec]
-                ;
-                Mess = 'Continuing: ~w, from non singleton list: ~w', MArgs = [Fst,Sec]
-            )
-        ),
-        debug_message_prefixed( Bogs, Mess, Prefixed ),
-        debug_message( Topic, Prefixed, MArgs )
-    ).
+    ( memberchk(sel_name(Trd),Bogs) ->
+          Mess = 'Continuing with: ~w as the: ~w. From list: ~w',
+          MArgs= [Fst,Trd,Sec]
+          ;
+          Mess = 'Continuing with: ~w from list: ~w',
+          MArgs= [Fst,Sec]
+    ),
+    debug_call_message_opts( Mess, MArgs, Message, Args, Bogs ),
+    debug_message( Topic, Message, Args ).
 
 debug_call_topic_enum( [], _I, _Len, _Topic ).
 debug_call_topic_enum( [H|T], I, Len, Topic ) :-
@@ -836,10 +853,9 @@ debug_call_topic_enum( [H|T], I, Len, Topic ) :-
     J is I + 1,
     debug_call_topic_enum( T, J, Len, Topic ).
 
-debug_call_topic_list_delim( '', _Topic, _Pfx, _Mess ).
-debug_call_topic_list_delim( ListName, Topic, Pfx, Mess ) :-
-    debug_message_prefixed( Pfx, Mess, Prefixed ),
-    debug_message( Topic, Prefixed, [ListName] ).
+debug_call_topic_list_delim( ListName, Topic, Std, Bogs ) :-
+     debug_call_message_opts( Std, [ListName], Mess, Args, Bogs ), 
+     debug_message( Topic, Mess, Args ).
 
 debug_call_topic_time_which_readable( Wch, Wchable ) :-
     debug_call_topic_time_which_readable_known( Wch, Wchable ),
@@ -884,6 +900,28 @@ debug_message_prefixed( [H|T], Standard, Prefixed ) :-
     debug_message_prefixed_atom( Pfx, Standard, Prefixed ).
 debug_message_prefixed( _, Standard, Standard ).
 
+debug_call_message_opts( Std, Srgs, Mess, Args, Opts ) :-
+     debug_call_pred_in_opts_mess( Std, Srgs, Pess, Args, Opts ),
+     debug_message_prefixed( Opts, Pess, Mess ).
+
+debug_call_pred_in_opts_mess( Std, Opt, Prefixed, Prgs, Bogs ) :-
+     en_list( Opt, Opts ),
+     ( debug_call_pred_in_opts(Pid, Bogs)  ->
+          Pfx = 'Predicate: ~w',
+          debug_message_prefixed_atom( Pfx, Std, Prefixed ),
+          Prgs = [Pid|Opts]
+          ;
+          Prefixed = Std,
+          Prgs = Opts
+     ).
+
+debug_call_pred_in_opts( Pid, Opts ) :-
+    memberchk( pred(Fun,Ar), Opts ),
+    !,
+    Fun/Ar = Pid.
+debug_call_pred_in_opts( Pid, Opts ) :-
+    memberchk( pred(Pid), Opts ).
+
 debug_message_prefixed_atom( Pfx, Standard, Prefixed ) :-
     sub_atom( Standard, 0, 1, Aft, Fst ),
     downcase_atom( Fst, Low ),
